@@ -91,14 +91,14 @@ def start_from_settings(settings, timeout=10):
             except socket.error:
                 pass
     else:
+        moz = None
         if not settings.has_key('JSBRIDGE_REPL_HOST'):
             settings['JSBRIDGE_REPL_HOST'] =  'localhost:4242'
         host, port = settings['JSBRIDGE_REPL_HOST'].split(':')
         port = int(port)
     
     network.create_network(host, port)
-    browser_window = JSObject(network.bridge, "Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator).getMostRecentWindow('')")
-    return browser_window
+    return moz
 
 
 def set_debug(settings):
@@ -106,48 +106,65 @@ def set_debug(settings):
     module_path = global_settings.module_path   
     settings['MOZILLA_PLUGINS'] += [os.path.join(module_path, 'xpi',
                                         'javascript_debugger-0.9.87.4-fx+tb+sb+sm.xpi'),
-                                    os.path.join(module_path, 'xpi', 'firebug-1.2.0b7-fx.xpi'),
-                                    os.path.join(module_path, 'xpi', 'chromebug-trunk'),
+                                    os.path.join(module_path, 'xpi', 'firebug-1.2.1-fx.xpi'),
+                                    # os.path.join(module_path, 'xpi', 'chromebug-trunk'),
                                     ]
-    settings['MOZILLA_CMD_ARGS'] += ['-jsconsole', '-chrome', 
-                                     'chrome://chromebug/content/chromebug.xul', 
-                                     '-p', 'chromebug', '-firefox']
-    rdf = open(os.path.join(module_path, 'xpi', 'chromebug-trunk', 'install.rdf.tpl.xml'), 'r').read()
-    rdf = rdf.replace('@FULLVERSION@', '0.1')
-    rdf = rdf.replace('<em:updateURL>@UPDATEPATH@/update.rdf</em:updateURL>\n', '')
-    rdf = rdf.replace('<em:homepageURL>@UPDATEPATH@/index.html</em:homepageURL>\n', '')
-    rdf = rdf.replace('<em:updateKey>ToBeReplaceByAValueFromMcCoy</em:updateKey>\n', '')
-    f = open(os.path.join(module_path, 'xpi', 'chromebug-trunk', 'install.rdf'), 'w')
-    f.write(rdf)
-    f.close()
-    return settings
-    
-def main():
-    parser = optparse.OptionParser()
-    parser.add_option("-s", "--settings", dest="settings",
-                      help="Settings file for jsbridge.", metavar="JSBRIDGE_SETTINGS_FILE")
-    parser.add_option("-b", "--binary", dest="binary",
-                      help="Binary path.", metavar=None)
-    parser.add_option("-d", "--default-profile", dest="default-profile",
-                      help="Default profile path.", metavar=None)
-    parser.add_option('-l', "--launch", dest="launch", action="store_true",
-                      help="Launch a new firefox instance.", metavar=None)
-    parser.add_option('-u', "--host", dest='host', 
-                      help="The hostname:port pairing to connect to mozrepl.")
-                      
-    parser.add_option("-z", "--debug",
-                      action="store_true", dest="debug", default=False,
-                      help="Run with firebug, chromebug, venkman, and jsconsole")
-    
-    (options, args) = parser.parse_args()
-    
-    settings_path = getattr(options, 'settings', None)
+    settings['MOZILLA_CMD_ARGS'] += ['-jsconsole', #'-chrome', 
+                                     #'chrome://chromebug/content/chromebug.xul', 
+                                     #'-p', 'chromebug', '-firefox'
+                                     ]
+    settings['MOZILLA_PREFERENCES']['extensions.checkCompatibility'] = False                                 
+                                     
+    # rdf = open(os.path.join(module_path, 'xpi', 'chromebug-trunk', 'install.rdf.tpl.xml'), 'r').read()
+    # rdf = rdf.replace('@FULLVERSION@', '0.1')
+    # rdf = rdf.replace('<em:updateURL>@UPDATEPATH@/update.rdf</em:updateURL>\n', '')
+    # rdf = rdf.replace('<em:homepageURL>@UPDATEPATH@/index.html</em:homepageURL>\n', '')
+    # rdf = rdf.replace('<em:updateKey>ToBeReplaceByAValueFromMcCoy</em:updateKey>\n', '')
+    # f = open(os.path.join(module_path, 'xpi', 'chromebug-trunk', 'install.rdf'), 'w')
+    # f.write(rdf)
+    # f.close()
+    return settings 
+
+def get_settings(settings_path=None):
     if settings_path is not None:
         settings_path = os.path.abspath(os.path.expanduser(settings_path))
         os.environ[settings_env] = settings_path
         
     settings = simplesettings.initialize_settings(global_settings, sys.modules[__name__],     
-                                                  local_env_variable=settings_env)
+                                                  local_env_variable=settings_env
+                                                  )
+    return settings
+    
+def start(settings=None, start_firefox=False):
+    if settings is None:
+        settings = get_settings()
+    if start_firefox:
+        settings['JSBRIDGE_START_FIREFOX'] = True
+    moz = start_from_settings(settings)
+    return moz
+
+parser = optparse.OptionParser()
+parser.add_option("-s", "--settings", dest="settings",
+                  help="Settings file for jsbridge.", metavar="JSBRIDGE_SETTINGS_FILE")
+parser.add_option("-b", "--binary", dest="binary",
+                  help="Binary path.", metavar=None)
+parser.add_option("-d", "--default-profile", dest="default-profile",
+                  help="Default profile path.", metavar=None)
+parser.add_option('-l', "--launch", dest="launch", action="store_true",
+                  help="Launch a new firefox instance.", metavar=None)
+parser.add_option("-z", "--debug",
+                  action="store_true", dest="debug", default=False,
+                  help="Run with firebug, chromebug, venkman, and jsconsole")
+parser.add_option("-a", "--showall", 
+                  action="store_true", dest="showall", default=False,
+                  help="Show all back_channel logger output.")
+
+def cli(shell=True, parser=parser, block=True, options=None):    
+    """Command line interface for jsbridge."""
+    if options is None:
+        (options, args) = parser.parse_args()
+    settings_path = getattr(options, 'settings', None)
+    settings = get_settings(settings_path)
     
     option_overrides = [('binary', 'MOZILLA_BINARY',),
                         ('profile', 'MOZILLA_PROFILE',),
@@ -161,28 +178,47 @@ def main():
             settings[override] = getattr(options, opt)
     
     if options.debug is True:
-        set_debug(settings)    
+        set_debug(settings)
+    
+    if options.showall:
+        import events
+        def showall(e, result):
+            print(e+": "+repr(result))
+        events.add_global_listener(showall)
+    
+    moz = start(settings)
+    
+    if block:
+        if shell:
+            try:
+                import IPython
+            except:
+                IPython = None    
+            if IPython is not None and '--usecode' not in sys.argv:
+                sys.argv = sys.argv[:1]
+                bridge = JSObject(network.bridge, "Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator).getMostRecentWindow('')")
+                ipython_shell({'bridge':bridge, 
+                               'getBrowserWindow':getBrowserWindow, 'getPreferencesWindow':getPreferencesWindow})#locals())
+            else:
+                code_shell({'bridge':bridge, 'getBrowserWindow':getBrowserWindow, 'getPreferencesWindow':getPreferencesWindow})
+            wait_function = None
+        elif moz:
+            wait_function = lambda : moz.wait()
+        else:
+            def wait_function():
+                while 1:
+                    sleep(.25)
+                    
+        if wait_function:
+            try:
+                wait_function()
+            except KeyboardInterrupt:
+                pass
         
-    bridge = start_from_settings(settings)
-    if settings.has_key('moz'):
-        # We want the moz object in the local ns or the shell to access
-        moz = settings['moz']
-    
-    try:
-        import IPython
-    except:
-        IPython = None    
-    if IPython is not None and '--usecode' not in sys.argv:
-        sys.argv = sys.argv[:1]
-        ipython_shell({'bridge':bridge, 'getBrowserWindow':getBrowserWindow, 'getPreferencesWindow':getPreferencesWindow})#locals())
+        if moz:
+            try:
+                moz.stop()
+            except:
+                pass
     else:
-        code_shell({'bridge':bridge, 'getBrowserWindow':getBrowserWindow, 'getPreferencesWindow':getPreferencesWindow})
-    
-    # # There is a bug in some of the traceback code IPython keeps around that causes a strange error here.
-    # # The workaround is to remove it from sys.modules
-    # x = sys.modules.pop('IPython', None)
-    # if x is not None:
-    #     del x
-    
-    if settings.has_key('moz'):
-        moz.stop()
+        return moz
